package sun.nio.ch;

import sun.misc.Cleaner;
import java.nio.ByteBuffer;

class IOVecWrapper
{
    private static final int BASE_OFFSET = 0;
    private static final int LEN_OFFSET;
    private static final int SIZE_IOVEC;
    private final AllocatedNativeObject vecArray;
    private final int size;
    private final ByteBuffer[] buf;
    private final int[] position;
    private final int[] remaining;
    private final ByteBuffer[] shadow;
    final long address;
    static int addressSize;
    private static final ThreadLocal<IOVecWrapper> cached;
    
    private IOVecWrapper(int size) {
        this.size = size;
        this.buf = new ByteBuffer[size];
        this.position = new int[size];
        this.remaining = new int[size];
        this.shadow = new ByteBuffer[size];
        this.vecArray = new AllocatedNativeObject(size * IOVecWrapper.SIZE_IOVEC, false);
        this.address = this.vecArray.address();
    }
    
    static IOVecWrapper get(int n) {
        Object value = (IOVecWrapper)IOVecWrapper.cached.get();
        if (value != null && value.size < n) {
            value.vecArray.free();
            value = null;
        }
        if (value == null) {
            value = new IOVecWrapper(n);
            Cleaner.create(value, new IOVecWrapper.Deallocator(value.vecArray));
            IOVecWrapper.cached.set(value);
            return value;
        }
        return value;
    }
    
    void setBuffer(int n, ByteBuffer byteBuffer, int n2, int n3) {
        this.buf[n] = byteBuffer;
        this.position[n] = n2;
        this.remaining[n] = n3;
    }
    
    void setShadow(int n, ByteBuffer byteBuffer) {
        this.shadow[n] = byteBuffer;
    }
    
    ByteBuffer getBuffer(int n) {
        return this.buf[n];
    }
    
    int getPosition(int n) {
        return this.position[n];
    }
    
    int getRemaining(int n) {
        return this.remaining[n];
    }
    
    ByteBuffer getShadow(int n) {
        return this.shadow[n];
    }
    
    void clearRefs(int n) {
        this.buf[n] = null;
        this.shadow[n] = null;
    }
    
    void putBase(int n, long n2) {
        int n3 = IOVecWrapper.SIZE_IOVEC * n + 0;
        if (IOVecWrapper.addressSize == 4) {
            this.vecArray.putInt(n3, (int)n2);
        }
        else {
            this.vecArray.putLong(n3, n2);
        }
    }
    
    void putLen(int n, long n2) {
        int n3 = IOVecWrapper.SIZE_IOVEC * n + IOVecWrapper.LEN_OFFSET;
        if (IOVecWrapper.addressSize == 4) {
            this.vecArray.putInt(n3, (int)n2);
        }
        else {
            this.vecArray.putLong(n3, n2);
        }
    }
    
    static {
        cached = new ThreadLocal();
        IOVecWrapper.addressSize = Util.unsafe().addressSize();
        LEN_OFFSET = IOVecWrapper.addressSize;
        SIZE_IOVEC = (short)(IOVecWrapper.addressSize * 2);
    }
    
    private static class Deallocator implements Runnable
    {
        private final AllocatedNativeObject obj;
        
        Deallocator(AllocatedNativeObject obj) {
            this.obj = obj;
        }
        
        @Override
        public void run() {
            this.obj.free();
        }
    }
}
